#!C:/Users/user/AppData/Local/Programs/Python/Python39/python.exe
#Replace the above location with yours. E.g #!/usr/bin/python3 for Linux
import requests
from bs4 import BeautifulSoup
import json
import re
from telegram import Update
from telegram.ext import Updater, CommandHandler, CallbackContext

def get_webpage(url):
    agent = {'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/75.0.3770.142 Safari/537.36', 'Accept-Encoding': 'gzip, deflate', 'Accept': '*/*', 'Connection': 'keep-alive'} # This is chrome, you can set whatever browser you like
    r = requests.get(url, headers=agent)
    if r.status_code != 200:
        print('Invalid url: {} {}'.format(r.status_code, r.url) )
        return ""
    else:
        return r.text

def get_weather(location) :
    # Get the content of web page
    the_url = "https://www.google.com/search?q=weather+accuweather+{}".format(location) 
    r_text = get_webpage(the_url)
    
    # Crawl and analyze
    # Get the url of accuweather from Google results
    soup = BeautifulSoup(r_text, "html.parser")
    google_results  = soup.find_all(href=re.compile("https://www.accuweather.com/"))
    
    # Get content from the url of accuweather
    weather_page = ""
    for a in google_results :
        if a['href'].find('https://www.accuweather.com/') == 0 and '/weather-forecast/' in a['href'] : 
            # e.g https://www.accuweather.com/en/de/berlin/10178/weather-forecast/178087
            the_url = a['href']
            weather_page = get_webpage(the_url)
            break
    
    # Ignore invalid location
    if weather_page == "" : 
        #return {"current": "N/A", "location": location, "temp" : "N/A"}  
        return ["N/A", location, "N/A"]          
    
    # Crawl and analyze the web page of accuweather
    soup = BeautifulSoup(weather_page, "html.parser")
    wcard = soup.find("a", attrs={"class" : "cur-con-weather-card"})
    current = soup.find("p", attrs={"class" : "cur-con-weather-card__subtitle"}).text
    temperature = wcard.find("div", attrs={"class" : "temp"}).text
    return [current.strip(), location, temperature] 
    
def handle_help(update: Update, context: CallbackContext) :
    # For commands of /start and /help
    text = update.effective_message.text
    print(text)
    # Show welcome only for command /start
    if text.split(" ")[0] == "/start" :
        # Get id of the chat room.
        chat_id = update.effective_chat.id
        url = "https://freepngimg.com/thumb/welcome/26893-3-welcome-free-download-thumb.png"
        context.bot.send_photo(chat_id, photo=url)
    
    # Reply help descryption
    update.message.reply_text("/temp location_name\n\nYou can get current temperature everywhere.\n\nFor example, \n\t\t\t/temp Berlin\n\t\t\t/temp New York")
    
def handle_temperature(update: Update, context: CallbackContext) :
    # For command /temp
    text = update.effective_message.text
    print(text)
    
    # Get arguments as location names
    if len(context.args) == 0 : 
        update.message.reply_text("/temp location_name\n\nMust has arguments.")
        return 
    loc = " ".join(context.args)

    # Send message to Telegram users.
    rlist = get_weather(loc)
    print(rlist)
    update.message.reply_text("\t\t\t".join(rlist))

# Daemon start    
if __name__ == "__main__":
    print("=== Weather Bot Started ===")
    updater = Updater('YOUR_TOKEN')
    dp = updater.dispatcher
    # Add handlers
    dp.add_handler( CommandHandler('start', handle_help) )
    dp.add_handler( CommandHandler('help', handle_help) )
    dp.add_handler( CommandHandler('temp', handle_temperature, pass_args=True) )
    # Listen requests to serve
    updater.start_polling()
    updater.idle()
